//   Delphi <-> FSUIPC/WideFS Communication                       Version 1.004
//   Copyright  2000 by Pelle F. S. Liljendal               pelle@liljendal.dk
//   All Rights Reserved
//
//   MainForm.pas
//
//   Example-program demostrating how to use FPCuser.pas (based on C-source
//   by Peter Dowson)
//
//   (Modified by Chris Brett to add support for FS2002 UIPC compatibility.)
//   (chris@formulate.clara.net)
//
//-----------------------------------------------------------------------------
unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, StdCtrls, ExtCtrls;

Const
  ResultText : Array[0..15] of String = ('Okay',
                                        'Attempt to Open when already Open',
                                        'Cannot link to FSUIPC or WideClient',
                                        'Failed to Register common message with Windows',
                                        'Failed to create Atom for mapping filename',
                                        'Failed to create a file mapping object',
                                        'Failed to open a view to the file map',
                                        'Incorrect version of FSUIPC, or not FSUIPC',
                                        'Sim is not version requested',
                                        'Call cannot execute, link not Open',
                                        'Call cannot execute: no requests accumulated',
                                        'IPC timed out all retries',
                                        'IPC sendmessage failed all retries',
                                        'IPC request contains bad data',
                                        'Maybe running on WideClient, but FS not running on Server, or wrong FSUIPC',
                                        'Read or Write request cannot be added, memory for Process is full');
  SimulationText : Array[0..6] of String = ('Any', 'FS98', 'FS2K', 'CFS2', 'CFS1', 'FLY', 'FS2002');


type
  TFormMain = class(TForm)
    Label1             : TLabel;
    LabelSim           : TLabel;
    Label2             : TLabel;
    Label3             : TLabel;
    LabelFSUIPCversion : TLabel;
    LabelFSclock       : TLabel;
    ButtonOk           : TButton;
    TimerUpdateTime: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonOkClick(Sender: TObject);
    procedure TimerUpdateTimeTimer(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormMain: TFormMain;

implementation

{$R *.DFM}

Uses
  FPCuser;


procedure TFormMain.FormCreate(Sender: TObject);
var
  dwResult : DWORD;
begin
  // Try to connect to FSUIPC (or WideFS)
  if FSUIPC_Open(SIM_ANY, dwResult) then begin
    // Connected to simulator (visible in the form's caption)
    FormMain.Caption           := FormMain.Caption + ResultText[dwResult];
    // Show what simulator we are connected to
    LabelSim.Caption           := SimulationText[FSUIPC_FS_Version];
    // Show version of FSUIPC
    LabelFSUIPCversion.Caption := Char(Ord('0') + ($0F and (FSUIPC_Version SHR 28))) +
                                  '.' +
                                  Char(Ord('0') + ($0F and (FSUIPC_Version SHR 24))) +
                                  Char(Ord('0') + ($0F and (FSUIPC_Version SHR 20))) +
                                  Char(Ord('0') + ($0F and (FSUIPC_Version SHR 16)));
    if (FSUIPC_Version and $FFFF) <> 0 then begin
      LabelFSUIPCversion.Caption := LabelFSUIPCversion.Caption  +
                                    Char(Ord('a') + (FSUIPC_Version and $FF) - 1);
    end;
    // Enable the timer which is actually reading the time
    TimerUpdateTime.Enabled := True;
  end else begin
    // Unable to "Connect"
    FormMain.Caption := FormMain.Caption + ResultText[dwResult];
  end;
end;


procedure TFormMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  // Disconnect from FSUIPC/WideFS (and dispose used "objects")
  FSUIPC_Close;
end;


procedure TFormMain.ButtonOkClick(Sender: TObject);
begin
  // Close the (Delphi) MainForm, hence end the application
  Close;
end;



procedure TFormMain.TimerUpdateTimeTimer(Sender: TObject);
var
  dwResult : DWORD;
  auiTime  : Array[1..3] of Byte;
begin
  // As an example of retrieving data, well also get the FS clock time too:
  // If we wanted additional reads/writes at the same time, we could put them here
  if FSUIPC_Read($0238, 3, @auiTime, dwResult) then begin
    // "Reed" proceeded without any problems
    if FSUIPC_Process(dwResult) then begin
      // "Process" proceeded withou any problems
      LabelFSclock.Caption := Format('%.2d:%.2d:%.2d', [auiTime[1], auiTime[2], auiTime[3]]);
    end else begin
      // Unable to "Process"
      LabelFSclock.Caption := 'Processing: ' + ResultText[dwResult];
    end;
  end else begin
    // Unable to "Read"
    LabelFSclock.Caption := 'Reading: ' + ResultText[dwResult];
  end;
end;

end.
